"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PubSub = void 0;
const tslib_1 = require("tslib");
const buffer_1 = require("buffer");
const uuid_1 = require("uuid");
const errors_1 = require("./errors");
class PubSub {
    constructor(ipfs, namespace) {
        this.ipfs = ipfs;
        this.namespace = namespace;
        this.subscribed = [];
    }
    create(namespace) {
        return new PubSub(this.ipfs, `${this.namespace}/${namespace}`);
    }
    getTopic(topic) {
        return `${this.namespace}.${topic}`;
    }
    encode(data) {
        return JSON.stringify(data);
    }
    decode(data) {
        return JSON.parse(data);
    }
    topics(ignoreInternals = true) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const topic = yield this.ipfs.pubsub.ls();
            return topic
                .map((topic) => topic.replace(`${this.namespace}.`, ''))
                .filter((value) => {
                if (ignoreInternals && value.startsWith('$$')) {
                    return false;
                }
                return true;
            });
        });
    }
    peers(topic) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const peers = yield this.ipfs.pubsub.peers(this.getTopic(topic));
            return peers.length;
        });
    }
    subscribe(topic, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.subscribed.push(topic);
            yield this.ipfs.pubsub.subscribe(this.getTopic(topic), (message) => {
                listener({
                    from: message.from,
                    data: this.decode(buffer_1.Buffer.from(message.data).toString())
                });
            });
        });
    }
    handleRequest(method, listener) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.subscribe(method, (msg) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const data = msg.data;
                listener(data.args, (data) => this.publish(data.reply, data));
            }));
        });
    }
    request(method, args, timeout = 1000) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const reply = (0, uuid_1.v4)();
            // eslint-disable-next-line no-async-promise-executor
            return new Promise((resolve, reject) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                yield this.subscribe(reply, (msg) => resolve(msg.data));
                setTimeout(() => reject(new errors_1.TimeoutError()), timeout);
                yield this.publish(method, { reply, args });
            }));
        });
    }
    waitForPeers(topic, depth = 0) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (depth > 100)
                throw new errors_1.TimeoutError();
            depth++;
            const peers = yield this.peers(topic);
            if (!peers) {
                console.debug('Waiting for peers to appear before publishing to', topic);
                yield new Promise((resolve) => setTimeout(resolve, 2 * depth * 10));
                return this.waitForPeers(topic, depth);
            }
        });
    }
    publish(topic, data, waitForPeers = true) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!waitForPeers)
                yield this.waitForPeers(topic);
            yield this.ipfs.pubsub.publish(this.getTopic(topic), buffer_1.Buffer.from(this.encode(data)));
        });
    }
    unsubscribe(topic) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            delete this.subscribed[this.subscribed.indexOf(topic)];
            yield this.ipfs.pubsub.unsubscribe(this.getTopic(topic));
        });
    }
    stop() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield Promise.all(this.subscribed.map(this.unsubscribe));
        });
    }
}
exports.PubSub = PubSub;
//# sourceMappingURL=pubsub.js.map