import type { IPFS } from 'ipfs-core-types';
import type { CID } from 'ipfs-core';
import { Store } from '.';
import { PubSub } from './pubsub';
import { Storage } from './storage';
export interface Peer {
    id: string;
    address: string;
}
export interface PeerAnnouncement {
    kind: 'announcement';
    peer: Peer;
}
export declare type StackPubSubMessage = PeerAnnouncement;
export interface StackOptions {
    namespace: string;
    /**
     * IPFS Instance
     *
     * No need to provide it unless you want custom IPFS instance to be used
     * `@dstack-js/ipfs` will be used by default
     */
    ipfs?: IPFS;
    /**
     * WebRTC implementation
     *
     * No need to provide it unless you want to use DStack in non browser environment
     */
    wrtc?: any;
    /**
     * Relay GraphQL Endpoint
     *
     * Defaults to DStack Cloud
     */
    relay?: string;
    /**
     * Storage implementation
     *
     * No need to provide it unless you want custom storage implementation to be used
     */
    storage?: Storage;
}
export declare class Stack {
    namespace: CID;
    ipfs: IPFS;
    pubsub: PubSub<StackPubSubMessage>;
    store: Store;
    private announceInterval?;
    announce: boolean;
    constructor(namespace: CID, ipfs: IPFS, storage: Storage);
    /**
     * get this peer info
     */
    id(): Promise<Peer>;
    private get libp2p();
    start(): Promise<void>;
    stop(): Promise<void>;
    /**
     * Create stack
     *
     * @returns Stack instance
     */
    static create({ namespace, ipfs, storage, relay, wrtc }: StackOptions): Promise<Stack>;
    /**
     * Get connected peers
     *
     * @returns connected peers list
     */
    peers(): Promise<Peer[]>;
    /**
     * Connect to peer
     *
     * By default IPFS will connect to some peers automatically, no need to use it without a reason
     *
     * @param address MultiAddr to connect
     */
    connect(address: string): Promise<void>;
    /**
     * Listen on new peer connected
     *
     * @param listener Will be called with peer info
     */
    onPeerConnect(listener: (peer: Peer) => void): void;
    /**
     * Listen on peer disconnected
     *
     * @param listener Will be called with peer info
     */
    onPeerDisconnected(listener: (peer: Peer) => void): void;
    /**
     * Ping peer
     *
     * @param peer destination
     * @param options timeout and cycles settings `{ timeout: 1000, count: 10 }`
     * @returns round-trip time in ms
     */
    ping(peer: Peer, { timeout, count }: {
        timeout?: number | undefined;
        count?: number | undefined;
    }): Promise<number>;
}
