"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stack = void 0;
const tslib_1 = require("tslib");
const it_all_1 = tslib_1.__importDefault(require("it-all"));
const _1 = require(".");
const pubsub_1 = require("./pubsub");
const storage_1 = require("./storage");
const ipfs_1 = require("@dstack-js/ipfs");
class Stack {
    constructor(namespace, ipfs, storage) {
        this.namespace = namespace;
        this.ipfs = ipfs;
        this.announce = false;
        this.pubsub = new pubsub_1.PubSub(ipfs, namespace.toString());
        this.store = new _1.Store(this, storage);
    }
    /**
     * get this peer info
     */
    id() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield this.ipfs.id();
            return {
                id: result.id,
                address: result.addresses[0].toString()
            };
        });
    }
    get libp2p() {
        return this.ipfs.libp2p;
    }
    start() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.announceInterval = setInterval(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (!this.announce)
                    return;
                yield this.pubsub.publish('announce', {
                    kind: 'announcement',
                    peer: yield this.id()
                });
            }), 250);
            yield this.store.start();
        });
    }
    stop() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.announceInterval)
                clearInterval(this.announceInterval);
            yield this.store.stop();
            yield this.pubsub.stop();
        });
    }
    /**
     * Create stack
     *
     * @returns Stack instance
     */
    static create({ namespace, ipfs, storage, relay, wrtc }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!ipfs) {
                ipfs = yield (0, ipfs_1.create)({ namespace, relay, wrtc });
            }
            const cid = yield ipfs.dag.put({ namespace });
            storage = storage || new storage_1.Storage(namespace);
            const stack = new Stack(cid, ipfs, storage);
            yield stack.start();
            return stack;
        });
    }
    /**
     * Get connected peers
     *
     * @returns connected peers list
     */
    peers() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const peers = yield this.ipfs.swarm.peers();
            return peers.map(({ peer, addr }) => ({
                id: peer,
                address: addr.toString()
            }));
        });
    }
    /**
     * Connect to peer
     *
     * By default IPFS will connect to some peers automatically, no need to use it without a reason
     *
     * @param address MultiAddr to connect
     */
    connect(address) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.ipfs.swarm.connect(address);
        });
    }
    /**
     * Listen on new peer connected
     *
     * @param listener Will be called with peer info
     */
    onPeerConnect(listener) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        this.libp2p.addressManager.on('peer:connect', (event) => {
            listener({
                id: event.remotePeer.toB58String(),
                address: event.remoteAddr.toString()
            });
        });
    }
    /**
     * Listen on peer disconnected
     *
     * @param listener Will be called with peer info
     */
    onPeerDisconnected(listener) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        this.libp2p.addressManager.on('peer:disconnect', (event) => {
            listener({
                id: event.remotePeer.toB58String(),
                address: event.remoteAddr.toString()
            });
        });
    }
    /**
     * Ping peer
     *
     * @param peer destination
     * @param options timeout and cycles settings `{ timeout: 1000, count: 10 }`
     * @returns round-trip time in ms
     */
    ping(peer, { timeout = 1000, count = 10 }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let ping = yield (0, it_all_1.default)(this.ipfs.ping(peer.id, { count, timeout }));
            ping = ping.filter(({ success }) => success);
            const result = ping.slice(-1)[0].text.match(/\d+.\d+ms/);
            if (!ping.length || !result)
                throw new _1.PeerUnreachableError();
            return parseFloat(result[0].split('ms')[0]);
        });
    }
}
exports.Stack = Stack;
//# sourceMappingURL=stack.js.map